######################################################################
#                                                                    #
#  This script constructs an indicator for marine bird productivity  #
#  for one species in one OSPAR region. In this case, black-legged   #
#        kittiwake in OSPAR region II, the Greater North Sea.        #
#              The script is built in R version 4.1.2                #
#                                                                    #
#             The script contains the following steps:               #
#                 Step 1: Load required R packages                   #
#                    Step 2: Import data files                       #
#       Step 3: Calculate and plot mean breeding productivity        #
#             Step 4: Construct basic population model               #
#                   Step 5: Load helper function                     #
#            Step 6: Tune model and construct indicator              #
#                                                                    #
######################################################################


####################################
# Step 1: Load required R packages #
####################################

require(readxl)
require(emmeans)
require(popbio)
require(visreg)


#############################
# Step 2: Import data files #
#############################

setwd("U:/NEA PANACEA/") # Adapt as required

# Import species list with custom species codes
# Constructed by Morten Frederiksen
spec <- read_xlsx("Species list.xlsx")

# Import file with raw breeding productivity data for all species and regions
# Extracted from ICES database by JNCC
data <- read_xlsx("Data/Breeding_success_Verified.xlsx")

# Import abundance indicator (B1) for region II
# File adapted from 'OSPARII_unsm_all.csv' supplied by JNCC
# Custom species codes added manually
abund_data <- read_excel("Data/OSPARII_abundance.xlsx")


#########################################################
# Step 3: Calculate and plot mean breeding productivity #
#########################################################

# 3.1 Select data and calculate variables

# Choose species
species <- spec[spec$species_code=="BLKI",] # Change as required

# Convert column to correct format
data$Chicks_fledged <- as.integer(data$Chicks_fledged)

# Use only lines with breeding productivity data
data <- subset(data,data$Plot_size>0&data$Chicks_fledged>-1)

# Select species, years and region
data <- subset(data,data$Species_name==species$species_name)
data <- subset(data,data$Year>1985)
data <- subset(data,data$Region_v3=="II") # Region II

# Calculate breeding productivity in each row
data$BP <- data$Chicks_fledged/data$Plot_size

# 3.2 Calculate and smooth time series of breeding productivity

# Tabulate number of study plots per year
plotn <- xtabs(~Year,data=data) 

# Calculate mean annual breeding productivity
model <- lm(BP ~ as.factor(Year),weights = Plot_size, data = data)

# Establish reference grid for marginal means
refgrid <- ref_grid(model)

# Get marginal means
means <- emmeans(refgrid,"Year")

# Prepare data frame with annual means etc
Years <- data.frame(Year=seq(min(data$Year),max(data$Year)))
meanBP <- as.data.frame(means)
meanBP <- merge(Years,meanBP,all=T)

# Derive overall mean breeding productivity (mean of means)
(mean(meanBP$emmean))
(sd(meanBP$emmean))

# Derive 6-year retrospective running mean
# And total variance (sampling + process)
# First, allocate vectors
BPind <- vector(length=length(meanBP$emmean)-5)
SEind <- vector(length=length(meanBP$emmean)-5)
# Then calculate the mean
for (j in 1:length(BPind)){
  BPind[j] <- mean(meanBP$emmean[j:(j+5)],na.rm=T)
  SEind[j] <- sqrt(var(meanBP$emmean[j:(j+5)]))
}

# 3.3 Produce figure of breeding productivity

plot(meanBP$Year,meanBP$upper.CL,type="n",ylab="Breeding productivity",xlab="Year",yaxs="i",xaxs="i",bty="l",xlim=c(min(meanBP$Year)-0.5,max(meanBP$Year+0.5)),ylim=c(0,max(meanBP$upper.CL,na.rm=T)),cex.axis=1.5,cex.lab=1.5,main=paste0(species$species_common,", region II"))
lines(meanBP$Year,meanBP$emmean,lwd=2,col="red")
points(meanBP$Year,meanBP$emmean,lwd=2,col="black")
text(meanBP$Year,meanBP$emmean-0.02,labels=plotn)
lines(meanBP$Year,meanBP$upper.CL,lwd=1,col="red",lty="dashed")
lines(meanBP$Year,meanBP$lower.CL,lwd=1,col="red",lty="dashed")
lines(meanBP$Year[6:length(meanBP$Year)],BPind,lwd=2,col="black")
legend("bottomleft",legend=c("Annual mean","Six-year running mean"),lty=1,lwd=2,col=c("red","black"),bty="n",cex=1)


############################################
# Step 4: Construct basic population model #
############################################

# Basic BLKI model
# Parameterised according to Horswill & Robinson (2015)
# Pre-breeding census, i.e. modelled at the start of the breeding season

# Define dimensions of matrix, i.e. number of age classes
m <- matrix(0,nrow=4,ncol=4)

# Define values of demographic variables
# Survival of each age class
s0 <- 0.79
s1 <- 0.854
s2 <- 0.854
s3 <- 0.854
sa <- 0.854

# Breeding propensity
gamma4 <- 0.9

# Fecundity
p <- 0.69

# Sex ratio
sigma <- 0.5

# Fertility, i.e. number of surviving female offspring per breeding-age female
f4 <- sigma*p*gamma4*s0

# Define matrix elements
m[1,4] <- f4
m[2,1] <- s1
m[3,2] <- s2
m[4,3] <- s3
m[4,4] <- sa

# Basic properties of the matrix
# Stable age distribution
(w <- stable.stage(m))
# Asymptotic growth rate
(lambda <- lambda(m))
# Generation time
(GT <- generation.time(m))


################################
# Step 5: Load helper function #
################################

# This function takes random draws from a beta distribution with a specified mean and SD.
# This is useful for stochastic modelling involving probabilities, where the random values
#  must be constrained to the interval 0-1
# Remember to run the lines below before step 6!

beta.draws <- function(p, sdp, draws) {
  varp <- sdp^2
  alpha <- p*(((p*(1-p))/varp) - 1)
  beta <- (1-p)*(((p*(1-p))/varp) - 1)
  rbeta(draws,alpha,beta)
}

###############################################
# Empirical data on SD of survival are sparse #
#  Values below are arbitrary, but realistic  #
#      Should work for most/all species       #
###############################################
sds0 <- 0.1
sds1 <- 0.1
sds2 <- 0.1
sds3 <- 0.1
sdsa <- 0.05 


##############################################
# Step 6: Tune model and construct indicator #
##############################################

# 6.1 Select abundance indicator data

ind_abund <- eval(parse(text = paste0("abund_data$",species$species_code)))

# 6.2 Re-calculate 6-year retrospective mean

# This calculation takes account of missing data
# Replace NAs in breeding productivity time series with mean of other years
overallmeanBP <- mean(meanBP$emmean,na.rm=T)
overallmeanSE <- mean(meanBP$SE,na.rm=T)
for(k in 1:nrow(meanBP)){
  if(is.na(meanBP$df[k])) meanBP$emmean[k] <- overallmeanBP
  if(is.na(meanBP$df[k])) meanBP$SE[k] <- overallmeanSE
}

# Calculate retrospective mean
BPind <- vector(length=length(meanBP$emmean)-5)
for (j in 1:length(BPind)){
  BPind[j] <- mean(meanBP$emmean[j:(j+5)],na.rm=T)
}

# 6.3 This is where values of age-specific survival are tuned

# %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
# Tune model to observed growth - adjust survival
s0 <- 0.49 # 0.79 # basic value
s1 <- 0.79 # 0.845 # basic value
s2 <- 0.854 # basic value
s3 <- 0.854 # basic value
sa <- 0.854 # basic value
# %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

# 6.4 Calculate stochastic growth rate from simulation model

# Number of simulations
sims <- 10000

# Create array to hold stochastic population projections
ww <- array(dim=c(nrow(meanBP)+1,length(w),sims))

# Initialise each simulation with stable age distribution, and project
for (i in 1:sims){
  ww[1,,i] <- w
  # Project with observed annual breeding success
  # Values drawn from normal distribution with observed mean and SE
  for(j in 1:nrow(meanBP)){
    m1 <- m
    # Fecundity
    p2 <- max(0,rnorm(1,meanBP$emmean[j],meanBP$SE[j]))
    # Fertility
    f4 <- sigma*p2*gamma4*s0
    # Define matrix elements
    m1[1,4] <- f4
    m1[2,1] <- s1
    m1[3,2] <- s2
    m1[4,3] <- s3
    m1[4,4] <- sa
    # Projection (matrix multiplication)
    ww[j+1,,i] <- m1 %*% ww[j,,i]
  }
}

# Extract breeding population
breedpop <- ww[,length(w),]

# Simulated stochastic lambda
# Calculated for each simulation from start and end values
simgrowth <- (breedpop[nrow(breedpop),]/breedpop[1,])^(1/(nrow(breedpop)-1))

# 6.5 Calculate observed growth rate from abundance indicator, and compare to simulated growth rate

# Use log-linear regression to estiamte observed growth rate 1991-2019
OYear <- 1991:2019
Index <- ind_abund
model <- lm(log(Index) ~ OYear)
out <- summary(model)

# Plot regression
visreg(model)

# Get distribution of observed growth rate, and back-transform to real scale
obsgrowth <- array(dim=sims)
realgrowth <- array(dim=sims)
for (i in 1:sims){
 obsgrowth[i] <- rnorm(1,out$coefficients[2,1],out$coefficients[2,2])
 realgrowth[i] <- exp(obsgrowth[i])
}

# Observed and simulated growth rates
mean(realgrowth)
mean(simgrowth)

# Compare simulated and observed growth rates graphically
par(mfrow=c(1,2))
boxplot(simgrowth,main="Simulated",ylab="Population growth rate",ylim=c(min(realgrowth),max(realgrowth)))
boxplot(realgrowth,main="Observed")

# 6.6 Tune the model

#%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
# Tune the model now! Return to step 6.3 and                
#  adjust survival until simulated and 
#  observed growth rates are the same.
#  Tolerance = 0.001
#%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

#%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
# Adjust fecundity so that growth rate = 1
#  Tolerance = 0.001 (if possible)
#%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

#fecundity
p2 <- 1.15 # 0.69 # basic value
#fertility
f4 <- sigma*p2*gamma4*s0
#define matrix elements
m1[1,4] <- f4

(lambda(m1))

# 6.7 Calculate thresholds for projected growth rate, with approximate confidence limits

# Calculate thresholds

# Generation time
(GTmean <- generation.time(m1))
# Thresholds for IUCN categories
(Threshold_VU <- (1-0.3)^(1/(3*GTmean)))
(Threshold_EN <- (1-0.5)^(1/(3*GTmean)))
(Threshold_CR <- (1-0.8)^(1/(3*GTmean)))


# Stochastic version of the baseline model
#  To get confidence limits for thresholds

# Prepare arrays
T_VU <- array(dim=sims)
T_EN <- array(dim=sims)
T_CR <- array(dim=sims)

# Run model with random draws for survival
for(j in 1:sims) {
 # Define dimensions of matrix
 m3 <- m1
 # Define demographic variables
 # Survival
 s0draw <- beta.draws(s0,sds0,1)
 s1draw <- beta.draws(s1,sds1,1)
 s2draw <- beta.draws(s2,sds2,1)
 s3draw <- beta.draws(s3,sds3,1)
 sadraw <- beta.draws(sa,sdsa,1)
 # Fertility
 f4 <- sigma*p2*gamma4*s0draw
 # Define matrix elements
 m3[1,4] <- f4
 m3[2,1] <- s1draw
 m3[3,2] <- s2draw
 m3[4,3] <- s3draw
 m3[4,4] <- sadraw
 # Generation time
 GT <- generation.time(m3)
 # Thresholds for IUCN categories
 T_VU[j] <- (1-0.3)^(1/(3*GT))
 T_EN[j] <- (1-0.5)^(1/(3*GT))
 T_CR[j] <- (1-0.8)^(1/(3*GT))
}

# Extract approximate confidence limits
T_VU_CL <- quantile(T_VU,probs=c(0.025,0.975),na.rm=T)
T_EN_CL <- quantile(T_EN,probs=c(0.025,0.975),na.rm=T)
T_CR_CL <- quantile(T_CR,probs=c(0.025,0.975),na.rm=T)

# 6.8 Calculate projected growth rates, i.e. indicator

# Prepare vector of projected growth rates
projlambda <- vector(mode="numeric",length=length(BPind))

# Loop over values of BPind (6-year mean breeding productivity)
#  to calculate indicator time series
for(i in 1:length(BPind)){
  # Define dimensions of matrix
  m2 <- m1
  # Fecundity
  p <- BPind[i]
  # Fertility
  f4 <- sigma*p*gamma4*s0
  # Define matrix elements
  m2[1,4] <- f4
  # Asymptotic growth rate
  projlambda[i] <- lambda(m2)
}

# 6.9 Plot projected growth rate and compare to thresholds

# Close previous graph window
graphics.off()

# Set up plot
pyear <- 1991:2020
par(mar=c(5.1,4.1,6.1,2.1),xpd=T)
plot(pyear,projlambda,type="n",bty="l",xlab="Year",ylab="Expected growth rate",ylim=c(min(min(projlambda),0.99),max(max(projlambda),1.01)),cex.axis=1.5,cex.lab=1.5,main=paste0(species$species_common,", region II"))
limits <- par("usr")
rect(limits[1]+0.001*(limits[2]-limits[1]),limits[3],limits[2],limits[4],col="green",border=NA)
if(Threshold_VU>limits[3])
  rect(limits[1]+0.001*(limits[2]-limits[1]),limits[3],limits[2],Threshold_VU,col="yellow",border=NA)
if(Threshold_EN>limits[3])
  rect(limits[1]+0.001*(limits[2]-limits[1]),limits[3],limits[2],Threshold_EN,col="orange",border=NA)
if(Threshold_CR>limits[3])
  rect(limits[1]+0.001*(limits[2]-limits[1]),limits[3],limits[2],Threshold_CR,col="red",border=NA)
lines(pyear,projlambda,lwd=2)
lines(pyear,projlambda,lwd=2)
legtext <- c("Above threshold","VU","EN","CR")
legend("top",legtext,fill=c("green","yellow","orange","red"),horiz=T,inset=c(0,-0.1),bty="n")
par(xpd=F)
abline(h=1)

# 6.10 Create output data set

# Indicator variables for thresholds (pass = 1)
PassVU <- ifelse(projlambda>Threshold_VU,1,0)
PassEN <- ifelse(projlambda>Threshold_EN,1,0)
PassCR <- ifelse(projlambda>Threshold_CR,1,0)

Results <- data.frame(pyear,BPind,projlambda,PassVU,PassEN,PassCR)
colnames(Results) <- c("Year","Mean BP","Expected_lambda","PassVU","PassEN","PassCR")
Results$Species <- species$species_common
Results$Guild <- species$feeding_guild
Results$Region <- "II"
Results$Threshold_VU <- Threshold_VU
Results$Threshold_EN <- Threshold_EN
Results$Threshold_CR <- Threshold_CR
Results$Generation_Time <- GTmean

# Show output data
Results

# Add code to open existing data set, append results, and save
# Something like
# Not run!
# load("OverallResults.Rdata")
# OverallResults <- rbind(OverallResults,Results)
# save(OverallResults,file="OverallResults.Rdata")
# Only works if 'OverallResults.Rdata' is created by another script
#  and is located in the right folder

# Alternatively, save only this example
# Not run!
# save(Results,file="Results BLKI reg II.Rdata")

# Convert to Excel
# Not run!
# require(writexl)
# write_xlsx(Results,"B3 Results/Results BLKI reg II.xlsx")

